// AForge Neural Net Library
//
// Copyright  Andrew Kirillov, 2005-2006
// andrew.kirillov@gmail.com
//

namespace NeuralNetworkPro.Activation_Functions
{
	using System;

	/// <summary>
	/// Activation function interface
	/// </summary>
	/// 
	/// <remarks>All activation functions, which are supposed to be used with
	/// neurons, which calculate their output as a function of weighted sum of
	/// their inputs, should implement this interfaces.
	/// </remarks>
	public interface IActivationFunction
	{
		/// <summary>
		/// Calculates function value
		/// </summary>
		///
		/// <param name="x">Function input value</param>
		/// 
		/// <returns>Function output value, <i>f(x)</i></returns>
		///
		/// <remarks>The method calculates function value at point <b>x</b>.</remarks>
		///
		double Function( double x );

		/// <summary>
		/// Calculates function derivative
		/// </summary>
		/// 
		/// <param name="x">Function input value</param>
		/// 
		/// <returns>Function derivative, <i>f'(x)</i></returns>
		/// 
		/// <remarks>The method calculates function derivative at point <b>x</b>.</remarks>
		///
		double Derivative( double x );

        string Name { get; }
	}
}
