﻿module Component
open Microsoft.Xna.Framework
open Microsoft.Xna.Framework.Graphics

type Camera =
  { world : Matrix ;
    view   : Matrix ;
    projection : Matrix}

type Component = 
                {update : float32 -> unit;
                 draw   : Camera -> unit;
                 load_content : unit}

type Time = { T : float32; DT : float32 }
let current_time = ref { T = 0.0f; DT = 0.0f }

type RenderState = 
  { mutable Text : (string * Vector2 * Vector2 * Color) option ;
    mutable Texture : (string * Vector2 * Vector2 * Color) option }

type Items =
  { sprite_batch : SpriteBatch;
    sprite_font : SpriteFont;
    renderer : RenderState;
    screen_size : Vector2 ;
    text_bar : Texture2D }

let build_items =
  (fun (game:Game) ->
    {
      sprite_batch =  new SpriteBatch(game.GraphicsDevice) ;
      sprite_font = game.Content.Load<SpriteFont>("font") ;
      renderer = {Text = None; Texture= None} ;
      screen_size = Vector2((float32 game.GraphicsDevice.Viewport.Width),
                            (float32 game.GraphicsDevice.Viewport.Height)) ;
      text_bar = game.Content.Load<Texture2D>("text_bar")
    })