﻿module X3D_FSharpClass
open System
open Microsoft.Xna.Framework
open Microsoft.Xna.Framework.Graphics
open Component


type Material (DEF: string, DiffuseColor: Vector3) =
  let mutable d_c = DiffuseColor
  member this.diffuseColor
      with get () = d_c
      and  set (newv) = d_c <- newv

type Appearance (Material: Material) = 
    member this.material = Material

type Box (DEF: string, Size: Vector3) = 
    let mutable myModel : Model = null
    let mutable sz = Size
    let mutable tr = Vector3.Zero
    let mutable ro = new Vector4(0.0f,0.0f,1.0f,0.0f)

    member this.rotation
        with get () = ro
        and  set (newv) = ro <- newv
    member this.translation
        with get () = tr
        and  set (newv) = tr <- newv
    member this.size
        with get () = sz
        and  set (newv) = sz <- newv

    member this.set_Model (newv) = 
        myModel <- newv    

    member this.Draw (camera:Camera,appearence:Appearance) =
        let q = Quaternion.CreateFromAxisAngle(new Vector3(ro.X, ro.Y, ro.Z), ro.W);
        let world =
                  Matrix.CreateScale(sz * 0.15f) *
                  Matrix.CreateFromQuaternion(q) *
                  Matrix.CreateTranslation(tr) * camera.world;
        
        let basic_effect = (myModel.Meshes.[0].MeshParts.[0].Effect :?> BasicEffect)
        basic_effect.EnableDefaultLighting()
        basic_effect.DiffuseColor <- appearence.material.diffuseColor
        basic_effect.World <- world
        basic_effect.View <- camera.view
        basic_effect.Projection <- camera.projection
        myModel.Meshes.[0].Draw()
//        for m in myModel.Meshes do
//            for eff in m.Effects do                
//                let basic_effect = (eff :?> BasicEffect)
//                basic_effect.EnableDefaultLighting()
//                basic_effect.DiffuseColor <- appearence.material.diffuseColor
//        myModel.Draw(world,camera.view,camera.projection)
                
type ColorInterpolator (DEF: string, Key: Vector3 list, KeyValue: float32 list) =
  let mutable f = KeyValue.Head
  let mutable v = Key.Head
  member this.set_fraction
      with get () = f
      and  set (newv:float32) = f <- Math.Min(1.0f,newv)
  member this.value_changed
      with get () =
        let d_index =(KeyValue |> List.findIndex(fun elem -> elem >= f))
        let mutable destination_keyvalue = KeyValue.Item d_index
        let s_index = (Math.Max(0, d_index - 1))
        let mutable start_keyvalue = KeyValue.Item s_index
        destination_keyvalue <- destination_keyvalue - start_keyvalue        
        let perc = (f - start_keyvalue) / (destination_keyvalue + 0.000001f)
        start_keyvalue <- 0.0f        
        Vector3.Lerp(Key.Item s_index,Key.Item d_index,perc)
      and  set (newv) =                 
        v <- newv

type PositionInterpolator (DEF: string, Key: Vector3 list, KeyValue: float32 list) =
  let mutable f = KeyValue.Head
  let mutable v = Key.Head
  member this.set_fraction
      with get () = f
      and  set (newv:float32) = f <- Math.Min(1.0f,newv)
  member this.value_changed
      with get () =
        let d_index =(KeyValue |> List.findIndex(fun elem -> elem >= f))
        let mutable destination_keyvalue = KeyValue.Item d_index
        let s_index = (Math.Max(0, d_index - 1))
        let mutable start_keyvalue = KeyValue.Item s_index
        destination_keyvalue <- destination_keyvalue - start_keyvalue        
        let perc = (f - start_keyvalue) / (destination_keyvalue + 0.000001f)
        start_keyvalue <- 0.0f        
        Vector3.Lerp(Key.Item s_index,Key.Item d_index,perc)
      and  set (newv) =                 
        v <- newv

type OrientationInterpolator (DEF: string, Key: Vector4 list, KeyValue: float32 list) =
  let mutable f = KeyValue.Head
  let mutable v = Key.Head
  member this.set_fraction
      with get () = f
      and  set (newv:float32) = f <- Math.Min(1.0f,newv)
  member this.value_changed
      with get () =
        let d_index =(KeyValue |> List.findIndex(fun elem -> elem >= f))
        let mutable destination_keyvalue = KeyValue.Item d_index
        let s_index = (Math.Max(0, d_index - 1))
        let mutable start_keyvalue = KeyValue.Item s_index
        destination_keyvalue <- destination_keyvalue - start_keyvalue        
        let perc = (f - start_keyvalue) / (destination_keyvalue + 0.000001f)
        start_keyvalue <- 0.0f        
        Vector4.Lerp(Key.Item s_index,Key.Item d_index,perc)
      and  set (newv) =                 
        v <- newv

type TimeSensor (Loop: bool, CycleInterval: float32) = 
  let mutable l =Loop
  let mutable ci = CycleInterval
  let mutable t = 0.0f

  member this.time 
      with get () = t
      and  set (newv) = t <- newv
  member this.fraction_changed
      with get () = (t % ci) / ci
      and  set (newv) = t <- newv

type Shape (Box: Box, Appearance: Appearance) =
    member this.Draw (camera:Camera) =
        Box.Draw(camera,Appearance)
    member this.box = Box

type Transform (shape :Shape,Translation: Vector3) =
    let mutable t = Translation
    let mutable r = new Vector4(0.0f,0.0f,0.0f,1.0f)

    member this.set_rotation
      with get () = r
      and  set (newv) = shape.box.rotation <- newv
    member this.set_translation
      with get () = t
      and  set (newv) = shape.box.translation <- newv




